//var userCenter = [45.3, 9.5];
//var userZoom = 2;

var map, delay, dgAttr, recent, vect, crosshair, selectionAreaBounds, searchControl;
var selection, areaSelect, mapCenter, areaSelectEnabled;

// This function gets called by Ruby after loading the saved center and zoom settings
function loadMap(userCenter, userZoom) {
    CONTROL_BAR_HEIGHT = 37;
    delay = 100;
    dgAttr = 'Map data &copy; <a href="#" onclick="openDGLink();">DigitalGlobe</a>';


    recent = L.tileLayer(dgUrl, {id: 'digitalglobe.nal0g75k', attribution: dgAttr});
    vect = L.tileLayer(glUrl, {
        id: 'cinvzo46s001ab4noph3c07n1',
        tileSize: 512,
        zoomOffset: -1,
        attribution: dgAttr
    });


    map = L.map(document.getElementById('map'), {
        center: userCenter,
        zoom: userZoom,
        maxZoom: 18,
        layers: [recent, vect]
    });

    crosshair = L.DomUtil.create("div", "selection-center", document.getElementById("map"));
    crosshair.setAttribute('hidden', true);

    selectionAreaBounds = L.DomUtil.create("div", "selection-bounds", document.getElementById("map"));
    selectionAreaBounds.setAttribute('hidden', true);
    selectionAreaBounds.style.zIndex = 999;

    searchControl = new L.Control.Search({
        url: 'https://nominatim.openstreetmap.org/search?format=json&q={s}',
        jsonpParam: 'json_callback',
        propertyName: 'display_name',
        propertyLoc: ['lat', 'lon'],
        container: 'SearchBox', //specify the document element id
        marker: false,
        collapsed: false,
        minLength: 2,
        zoom: 14
    });

    map.addControl(searchControl);
    areaSelectEnabled = false;

    map.on('zoomend', function() {
        disabledButtonsBasedOnZoom();
    });

    saveCurrentMapSettings(); //run this on load so the settings are based on the map
    SetSelectionBounds();

    map.on('move', function () {
        saveCurrentMapSettings();
        SetSelectionBounds();
    });

    disabledButtonsBasedOnZoom(); //run this so on load the button is enabled or disabled
}


function openDGLink() {
    window.location = 'skp:open_dg_link';
}

function openLeafletLink() {
    window.location = 'skp:open_leaflet_link';
}

function drawSelectionBox() {
    crosshair.removeAttribute("hidden");

    var w = window.innerWidth
        || document.documentElement.clientWidth
        || document.body.clientWidth;

    var h = window.innerHeight
        || document.documentElement.clientHeight
        || document.body.clientHeight;

    var width = Math.round(w - 30);
    var height = Math.round((h - CONTROL_BAR_HEIGHT) - 30);

    areaSelect = L.areaSelect({width:width, height:height});
    areaSelect.addTo(map);
    areaSelectEnabled = true;
    SetSelectionBounds();

    $('#SelectArea').hide();
    $('#ImportArea').show();
    $('#CancelArea').show();
}

function removeSelectionBox() {
    if (areaSelectEnabled){
        crosshair.setAttribute('hidden', true);
        areaSelectEnabled = false;
        areaSelect.remove();

        $('#SelectArea').show();
        $('#ImportArea').hide();
        $('#CancelArea').hide();
    }
}

function disabledButtonsBasedOnZoom() {
    var zoom_level = map.getZoom();
    if (zoom_level < 13){
        $('#ImportArea').attr("disabled", true);
        $('#SelectArea').attr("disabled", true);
    }else{
        $('#ImportArea').attr("disabled", false);
        $('#SelectArea').attr("disabled", false);
    }
}



function saveCurrentMapSettings() {
    document.getElementById('zoom').value = map.getZoom();
    var center = map.getCenter();
    document.getElementById('lat').value = center.lat;
    document.getElementById('lng').value = center.lng;
}

function importSelectionBox() {
    var w = window.innerWidth
        || document.documentElement.clientWidth
        || document.body.clientWidth;

    var h = window.innerHeight
        || document.documentElement.clientHeight
        || document.body.clientHeight;

    var topLeftX = Math.round((w - areaSelect._width) / 2);
    var topLeftY = Math.round(((h - CONTROL_BAR_HEIGHT) - areaSelect._height) / 2);
    var bottomRightX = w - topLeftX;
    var bottomRightY = h - topLeftY;

    bounds = areaSelect.getBounds();
    surface = [[bounds.getEast(), bounds.getSouth()], [bounds.getWest(), bounds.getSouth()], [bounds.getWest(), bounds.getNorth()], [bounds.getEast(), bounds.getNorth()]];
    areaSelectEnabled = false;
    areaSelect.remove();
    //hide/remove all controls from the leaflet map
    map.removeControl(map.zoomControl);
    map.removeControl(map.attributionControl);
    map.removeLayer(vect);
    crosshair.setAttribute('hidden', true);
    var image_location = {"top_left_x": topLeftX,
        "top_left_y": (topLeftY + CONTROL_BAR_HEIGHT),
        "bottom_right_x": bottomRightX,
        "bottom_right_y": bottomRightY,
        "center": map.getCenter(),
        "surface": surface,
        "zoom": map.getZoom()
    };

    setTimeout(function() {
        window.location = 'skp:set_location@' + JSON.stringify(image_location);
    }, delay);
}

//calculates the max selection area for the current window
//similar logic to what SU uses
function MaxSelectionArea() {
    var meter_to_2km = 2E3;
    var map_bounds = map.getBounds();
    var width_in_meters = map_bounds.getNorthWest().distanceTo(map_bounds.getNorthEast());
    if(width_in_meters > 1){
        var map_container_size = map.getSize(); //x and y value in pixels
        return Math.floor(meter_to_2km * map_container_size.x / width_in_meters) //return value is pixel per 5km
    }
}

//run when areaSelect is added and on move/zoom change
function limitAreaSelect(value){
    if (areaSelectEnabled){
        dimensions = {
            width: value,
            height: value
        }
        areaSelect.setMaxDimensions(dimensions)
    }
}

function SetSelectionBounds() {
    var zoom_level = map.getZoom();
    if (zoom_level >= 13) {
        var border_size = 3;
        selectionAreaBounds.style.border = border_size + "px solid #ffffff";
        var container_size = map.getSize();
        var pixel_per_distance = MaxSelectionArea();
        var left_position = (container_size.x - pixel_per_distance) / 2 - border_size;
        var top_position = (container_size.y - pixel_per_distance) / 2 - border_size;
        selectionAreaBounds.style.width = pixel_per_distance + "px";
        selectionAreaBounds.style.height = pixel_per_distance + "px";
        limitAreaSelect(pixel_per_distance);
        selectionAreaBounds.style.left = left_position + "px";
        selectionAreaBounds.style.top = top_position + "px";
        selectionAreaBounds.removeAttribute("hidden");
    }else{
        removeSelectionBox();
        selectionAreaBounds.setAttribute("hidden", true);
    }
}

