/**
 * Created by Brian on 20/09/2016.
 */
var expanded = {};
var l10n_strings = {};
var delay = 100;
var default_refresh_src = "../images/refresh-preview.svg";

var sections = [
    {
        name: 'Imagery',
        localizedName: l10n("Imagery"),
        localizedTitle: l10n("Import Map Tiles"),
        icon: 'imagery.svg'
    },
    {
        name: 'Roads',
        localizedName: l10n("Roads"),
        localizedTitle: l10n("Import OSM Road Data"),
        icon: 'roads.svg'
    },
    {
        name: 'Paths',
        localizedName: l10n("Paths"),
        localizedTitle: l10n("Import OSM Path Data"),
        icon: 'paths.svg'
    },
    {
        name: 'Buildings',
        localizedName: l10n("Buildings"),
        localizedTitle: l10n("Import OSM Building Data"),
        icon: 'buildings.svg'
    },
    {
        name: 'Water',
        localizedName: l10n("Water"),
        localizedTitle: l10n("Import OSM Water Bodies"),
        icon: 'water.svg'
    },
    {
        name: 'Trees',
        localizedName: l10n("Trees"),
        localizedTitle: l10n("Import OSM Tree Locations"),
        icon: 'tree.svg'
    }
];

var im_section = false;
var import_hash = {};

function addHeader(){
    document.write(PlaceMaker.header());
}

function addImportActions(){
    document.write(PlaceMaker.importActions());
    sections.forEach(add_section_to_html);
}

function addSubSections() {
    window.location = 'skp:load_prefs_and_listeners';
}

function expand(section) {
    toggleVisible(section);
    toggleChevronRotation(section);
}

function toggle3DOptionsVisible() {
    var options = document.querySelector('.volume-building-options');
    toggleVisible(options);
}

function toggleHeightOptionsVisible() {
    var options = document.querySelector('.random-height-options');
    toggleVisible(options);
}

function toggleVisible(node) {
    $(node).slideToggle();
}

function toggleChevronRotation(section) {
    var importSection = section.parentNode;
    var chevron = importSection.querySelector('.expand-switch');
    var classString = chevron.className;

    if (classString.search(/(^| )expanded($| )/) != -1) {
        classString = classString.replace(/(^| )expanded($| )/, '');
        chevron.className = classString;
    } else {
        classString += ' expanded';
        chevron.className = classString;
    }
}

function setStatus(sectionName, status, message) {
    var section = getImportSection(sectionName);
    var images = section.querySelectorAll('.status img');
    for (var i = 0; i < images.length; ++i) {
        hideElement(images[i]);
    }
    var statusImage;
    switch (status) {
        case 'waiting':
            statusImage = section.querySelector('.wait img');
            break;
        case 'done':
            statusImage = section.querySelector('.done img');
            break;
        case 'error':
            statusImage = section.querySelector('.error img');
            break;
    }
    showElement(statusImage);
    statusImage.title = message;
}
function resetStatus(sectionName) {
    var section = getImportSection(sectionName);
    var statusImage = section.querySelector('.wait img');
    hideElement(statusImage);
    statusImage = section.querySelector('.done img');
    hideElement(statusImage);
    statusImage = section.querySelector('.error img');
    hideElement(statusImage);
}

function showErrorMessage(node, message) {
    node.innerHTML = message;
}


function collapseAll() {
    var containers = document.getElementsByClassName('options-container');
    var nodeArray = Array.apply(null, containers);
    nodeArray.forEach(function(container) {
        hideElement(container);
    });
}

function addListeners() {
    var imageryButton = document.querySelector('.Imagery-section .import-button');
    imageryButton.addEventListener('click', importImagery);
    var imageryOptions = document.querySelector('.Imagery-section .options-container');
    document.querySelector('.Imagery-section .expandable-container').
        addEventListener('click', expand.bind(this, imageryOptions));
    document.querySelector('.preview-pane').
        addEventListener('click', updatePreview);

    var roadsButton = document.querySelector('.Roads-section .import-button');
    roadsButton.addEventListener('click', importRoads);
    var roadsOptions = document.querySelector('.Roads-section .options-container');
    document.querySelector('.Roads-section .expandable-container').
        addEventListener('click', expand.bind(this, roadsOptions));

    var pathsButton = document.querySelector('.Paths-section .import-button');
    pathsButton.addEventListener('click', importPaths);
    var pathsOptions = document.querySelector('.Paths-section .options-container');
    document.querySelector('.Paths-section .expandable-container').
        addEventListener('click', expand.bind(this, pathsOptions));

    var buildingsButton = document.querySelector('.Buildings-section .import-button');
    buildingsButton.addEventListener('click', importBuildings);
    var buildingsOptions = document.querySelector('.Buildings-section .options-container');
    document.querySelector('.Buildings-section .expandable-container').
        addEventListener('click', expand.bind(this, buildingsOptions));

    var waterButton = document.querySelector('.Water-section .import-button');
    waterButton.addEventListener('click', importWater);
    var waterOptions = document.querySelector('.Water-section .options-container');
    document.querySelector('.Water-section .expandable-container').
        addEventListener('click', expand.bind(this, waterOptions));

    var treesButton = document.querySelector('.Trees-section .import-button');
    treesButton.addEventListener('click', importTrees);
    var treesOptions = document.querySelector('.Trees-section .options-container');
    document.querySelector('.Trees-section .expandable-container').
        addEventListener('click', expand.bind(this, treesOptions));

    var importAllButton = document.querySelector('.import-all-button');
    importAllButton.addEventListener('click', importAll.bind(this));

    var saveSettingsButton = document.querySelector('.save-preferences-link');
    saveSettingsButton.addEventListener('click', saveSettings.bind(this));

    var geolocateButton = document.querySelector('.geolocate-button');
    geolocateButton.addEventListener('click', openGeoWindow.bind(this));

    var treeLibraryButton = document.querySelector('.tree-library-button');
    treeLibraryButton.addEventListener('click', selectTreeLibrary.bind(this));

    var make3DCheckbox = document.querySelector('.make3D');
    make3DCheckbox.addEventListener('click', toggle3DOptionsVisible.bind(this));

    var randomHeightCheckbox = document.querySelector('.randomize-height');
    randomHeightCheckbox.addEventListener('click', toggleHeightOptionsVisible.bind(this));

    var osmTerms = document.querySelector('.osm-terms-link');
    osmTerms.addEventListener('click', openTerms.bind(this, 'osm'));

    var drapeRoads = document.querySelector('.drape-roads');
    drapeRoads.addEventListener('click', validateDrape.bind(this, 'drape-roads'));

    var mergePaths = document.querySelector('.merge-paths');
    mergePaths.addEventListener('click', validateDrape.bind(this, 'merge-paths'));

    var dropBuildings = document.querySelector('.drop-buildings');
    dropBuildings.addEventListener('click', validateDrape.bind(this, 'drop-buildings'));

    var dropTrees = document.querySelector('.drop-trees');
    dropTrees.addEventListener('click', validateDrape.bind(this, 'drop-trees'));

    var dropWater = document.querySelector('.merge-water');
    dropWater.addEventListener('click', validateDrape.bind(this, 'merge-water'));
    
    var buildingService = document.querySelector('.building-service');
    buildingService.addEventListener('click', buyMoreBuildings.bind(this));

    var imagerySource = document.querySelector('.source');
    imagerySource.addEventListener('change', updateImagerySection.bind(this));
}

function buyMoreBuildings() {
    window.location = 'skp:buyMoreBuildings';
}

function updateImagerySection() {
    var imagerySource = document.querySelector('.source');
    var mapChoice = document.querySelector('.map-type-row');
    if (imagerySource.value == 'Nearmap') {
        mapChoice.style.display = 'none';
    } else {
        mapChoice.style.display = 'block'
    }
    var preview = document.querySelector('.preview-pane');
    enableElement(preview);
    var icon = preview.querySelector('.refresh-icon');
    showElement(icon);
    hideElement(preview.querySelector('.spinner'));
    icon.src = default_refresh_src;
}

function validateDrape(className) {
    window.location = 'skp:validate_drape@' + className;
}

function updatePreview() {
    var preview = document.querySelector('.preview-pane');
    disableElement(preview);
    hideElement(preview.querySelector('.refresh-icon'));
    showElement(preview.querySelector('.spinner'));
    options = getImageryOptions();
    setTimeout(function() {
        window.location = 'skp:update_preview@' + JSON.stringify(options);
    }, delay);
}

function resetPreview() {
    var preview = document.querySelector('.preview-pane');
    disableElement(preview);
    showElement(preview.querySelector('.refresh-icon'));
    hideElement(preview.querySelector('.spinner'));
}

function setImagePreview(src) {
    var preview = document.querySelector('.preview-pane');
    enableElement(preview);
    var icon = preview.querySelector('.refresh-icon');
    showElement(icon);
    hideElement(preview.querySelector('.spinner'));
    icon.src = src;
}

function importAll() {
    var importAllButton = document.querySelector('.import-all-button');
    disableButton(importAllButton);
    var allOptions = {
        Imagery: getImageryOptions(),
        Buildings: getBuildingsOptions(),
        Roads: getRoadsOptions(),
        Trees: getTreesOptions(),
        Water: getWaterOptions(),
        Paths: getPathsOptions()
    };
    startImport('Roads');
    startImport('Paths');
    startImport('Water');
    startImport('Trees');
    startImport('Buildings');
    setTimeout(function() {
        window.location = 'skp:import_all@' + JSON.stringify(allOptions);
    }, delay);
}

function saveSettings() {
    var allOptions = {
        Imagery: getImageryOptions(),
        Buildings: getBuildingsOptions(),
        Roads: getRoadsOptions(),
        Trees: getTreesOptions(),
        Water: getWaterOptions(),
        Paths: getPathsOptions()
    };
    setTimeout(function() {
        window.location = 'skp:save_as_defaults@' + JSON.stringify(allOptions);
    }, delay);
}

function openGeoWindow() {
    window.location = 'skp:open_geo_window';
}

function validateSurface() {
    window.location = 'skp:validate_surface';
}

function selectTreeLibrary() {
    window.location = 'skp:select_tree_library';
}

function setTreeLibrary(text) {
    var treeLibraryInput = document.querySelector('.tree-library input');
    treeLibraryInput.value = text;
}

function importImagery() {
    options = getImageryOptions();
    window.location = 'skp:import_high_resolution_aerial@' + JSON.stringify(options);
}

function importRoads() {
    startImport('Roads');
    // We include a slight delay to allow for manipulating and refreshing of the DOM
    setTimeout(function() {
    window.location = 'skp:import_roads@' + JSON.stringify(getRoadsOptions());
    }, delay);

}

function importPaths() {
    startImport('Paths');
    setTimeout(function() {
        window.location = 'skp:import_paths@' + JSON.stringify(getPathsOptions());
    }, delay);
}

function importWater() {
    startImport('Water');
    setTimeout(function() {
        window.location = 'skp:import_water@' + JSON.stringify(getWaterOptions());
    }, delay);
}

function importTrees() {
    startImport('Trees');
    window.location = 'skp:import_trees@' + JSON.stringify(getTreesOptions());
}

function importBuildings() {
    startImport('Buildings');
    setTimeout(function() {
        window.location = 'skp:import_buildings@' + JSON.stringify(getBuildingsOptions());
    }, delay);
}

function startImport(sectionName) {
    var importButton = getImportButton(sectionName);
    disableButton(importButton);
    setStatus(sectionName, 'waiting', '');
}

function resetImport(sectionName) {
    resetStatus(sectionName);
    var importButton = getImportButton(sectionName);
    enableButton(importButton);
}

function resetImportAll() {
    enableButton(document.querySelector('.import-all-button'));
}

function displayImportError(sectionName, message) {
    setStatus(sectionName, 'error', message);
    enableButton(getImportButton(sectionName));
}

function importComplete(sectionName, message) {
    var importButton = getImportButton(sectionName);
    enableButton(importButton);
    setStatus(sectionName, 'done', message);
}

function getImportButton(sectionName) {
    var section = getImportSection(sectionName);
    return section.querySelector('.import-button');
}

function getImportSection(sectionName) {
    return document.querySelector('.' + sectionName + '-section');
}

function getImageryOptions() {
    options = {
        source: document.getElementsByClassName('source')[0].value,
        zoom: document.getElementsByClassName('zoom')[0].value,
        materialEnabled: document.getElementsByClassName('paint-selected')[0].checked,
        mapType: document.getElementsByClassName('map-type')[0].value
    };
    return options;
}

function getRoadsOptions() {
    var options = {
        defaultWidth: encode(document.querySelector('.road-width-input input').value),
        serviceWidth: encode(document.querySelector('.service-road-width-input input').value),
        residentialWidth: encode(document.querySelector('.residential-road-width-input input').value),
        highwayWidth: encode(document.querySelector('.highway-road-width-input input').value),
        motorwayWidth: encode(document.querySelector('.motorway-road-width-input input').value),
        laneWidth: encode(document.querySelector('.lane-width-input input').value),
        drape: document.querySelector('.drape-roads').checked
    };
    return options;
}

function getPathsOptions() {
    var options = {
        defaultWidth: encode(document.querySelector('.path-width-input input').value),
        drape: document.querySelector('.merge-paths').checked
    };
    return options;
}

function getWaterOptions() {
    var options = {
        drape: document.querySelector('.merge-water').checked
    };
    return options;
}

function getTreesOptions() {
    var options = {
        library: encode(document.querySelector('.Trees-section .tree-library input').value),
        minHeight: encode(document.querySelector('.min-tree-height input').value),
        maxHeight: encode(document.querySelector('.max-tree-height input').value),
        spacing: encode(document.querySelector('.tree-spacing input').value),
        drape: document.querySelector('.drop-trees').checked
    };
    return options;
}

function getBuildingsOptions() {
    var options = {
        defaultHeight: encode(document.querySelector('.building-height-input input').value),
        levelHeight: encode(document.querySelector('.level-height-input input').value),
        make3D: document.querySelector('.make3D').checked,
        randomHeight: document.querySelector('.randomize-height').checked,
        materialEnabled: document.querySelector('.import-building-material').checked,
        drape: document.querySelector('.drop-buildings').checked,
        minHeight: encode(document.querySelector('.min-height-input input').value),
        maxHeight: encode(document.querySelector('.max-height-input input').value)
    };
    return options;
}

function addImageryOptions(defaults) {
    var imagery = document.querySelector('.Imagery-section');
    var imageryOptions = goog.soy.renderAsFragment(PlaceMaker.imagery, defaults);
    imagery.appendChild(imageryOptions);
}

function addRoadsOptions(defaults) {
    var roads = document.querySelector('.Roads-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.roads, defaults);
    roads.appendChild(options);
}

function addPathsOptions(defaults) {
    var paths = document.querySelector('.Paths-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.paths, defaults);
    paths.appendChild(options);
}

function addBuildingsOptions(defaults) {
    var section = document.querySelector('.Buildings-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.buildings, defaults);
    section.appendChild(options);
}

function addTreesOptions(defaults) {
    var section = document.querySelector('.Trees-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.trees, defaults);
    section.appendChild(options);
}

function addWaterOptions(defaults) {
    var section = document.querySelector('.Water-section');
    var options = goog.soy.renderAsFragment(PlaceMaker.water, defaults);
    section.appendChild(options);
}

function openTerms(type) {
    window.location = 'skp:open_terms@' + type;
}

function add_section_to_html(value, index) {
    document.write(PlaceMaker.importSection(value));
}

function addAttribution() {
    document.write(PlaceMaker.attribution());
}


